!
Subroutine rttov_profout_k( &
            nfrequencies, &     ! in
            nchannels, &        ! in
            nbtout, &           ! in
            nprofiles, &        ! in
            channels, &         ! in
            lprofiles, &        ! in
            polarisations, &    ! in
            coef, &             ! in
            geometry, &         ! in
            profiles_k_all, &   ! in
            profiles_k)         ! Out
  ! Description:
  ! To convert an K-matrix brightness temperatures with 1, 2 or 4 polarisations
  ! polarisation requested by the user.
  ! There are seven options:
  ! 0. Return average of V and H polarisation.
  ! 1. Return AMSU-style mix polarisation (nominal V at nadir)
  ! 2. Return AMSU-style mix polarisation (nominal H at nadir)
  ! 3. Return Vertical polarisation
  ! 4. Return Horizontal polarisation
  ! 5. Return vertical and horizontal polarisation
  ! 6. Return full Stokes vector
  !
  ! For IR channels this variable is not required, and one unpolarised brightness
  ! temperature is computed.
  !
  ! Note options 0-4 return one polarisation per channel. Option 5 returns
  ! 2 polarisations per channel and option 6 four polarisations per channel.
  ! Note also that for options 1-3 two polarisations must be computed in RTTOV,
  ! even though only one is returned. For this reason rad%bt is replaced by
  ! rad%out, where rad%out has length of number of output channels, whereas
  ! rad%bt has length of all brightness temperatures computed in RTTOV.
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2003, EUMETSAT, All Rights Reserved.
  !
  ! Method: Uses band correction coefficients for each channel
  !         read in from RT coefficient file.
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  ! 1.0    10/07/2003  New code required for polarimetric RTTOV (Steve English)
  ! 1.1    13/10/2006  Corrected bug in pol_id (R Saunders)
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Type Definitions:

  Use rttov_const, only :   &
       npolar_return,       &
       npolar_compute,      &
       pol_v ,              &
       pol_h

  Use rttov_types, Only : &
       rttov_coef     ,&
       profile_Type   ,&
       geometry_Type

  Use parkind1, Only : jpim     ,jprb
  Implicit None

  !subroutine arguments:
  Integer(Kind=jpim),  Intent(in)            :: nbtout
  Integer(Kind=jpim),  Intent(in)            :: nchannels
  Integer(Kind=jpim),  Intent(in)            :: nfrequencies
  Integer(Kind=jpim),  Intent(in)            :: nprofiles
  Type(geometry_Type), Intent(in) ,Target    :: geometry(nprofiles)
  Integer(Kind=jpim),  Intent(in)            :: channels(nfrequencies)
  Integer(Kind=jpim),  Intent(in)            :: polarisations(nchannels,3)
  Integer(Kind=jpim),  Intent(in)            :: lprofiles(nfrequencies)
  Type(rttov_coef),    Intent(in)            :: coef           ! Coefficients
  Type(profile_Type),  Intent(inout) ,Target :: profiles_k(nbtout)
  Type(profile_Type),  Intent(inout) ,Target :: profiles_k_all(nchannels)

  ! radiances are expressed in mw/cm-1/ster/sq.m
  ! and temperatures in Kelvin

  !local variables:
  Real(Kind=jprb) :: emissfactor_h,emissfactor_v
  Integer(Kind=jpim) :: chan,i,j,ich,ich2,pol_id,ii
  !Type(geometry_Type), Pointer    :: geom

  !- End of header ------------------------------------------------------

   ich2=1
   Do i=1,nfrequencies
      chan        = channels(i)
      pol_id      = 0
      pol_id = coef % fastem_polar(chan) + 1
      ich = polarisations(i,1)
      If (pol_id >= 4) then
         ! Return all calculated polarisations (or just computed TB for IR channels)
         Do j=1,polarisations(i,3)
            profiles_k(ich2+j-1) % s2m % t           = profiles_k_all(ich+j-1) % s2m % t
            profiles_k(ich2+j-1) % s2m % q           = profiles_k_all(ich+j-1) % s2m % q
            profiles_k(ich2+j-1) % s2m % p           = profiles_k_all(ich+j-1) % s2m % p
            profiles_k(ich2+j-1) % s2m % u           = profiles_k_all(ich+j-1) % s2m % u
            profiles_k(ich2+j-1) % s2m % v           = profiles_k_all(ich+j-1) % s2m % v
            profiles_k(ich2+j-1) % skin % t          = profiles_k_all(ich+j-1) % skin % t
            profiles_k(ich2+j-1) % skin % fastem(1)  = profiles_k_all(ich+j-1) % skin % fastem(1)
            profiles_k(ich2+j-1) % skin % fastem(2)  = profiles_k_all(ich+j-1) % skin % fastem(2)
            profiles_k(ich2+j-1) % skin % fastem(3)  = profiles_k_all(ich+j-1) % skin % fastem(3)
            profiles_k(ich2+j-1) % skin % fastem(4)  = profiles_k_all(ich+j-1) % skin % fastem(4)
            profiles_k(ich2+j-1) % skin % fastem(5)  = profiles_k_all(ich+j-1) % skin % fastem(5)
            profiles_k(ich2+j-1) % ctp               = profiles_k_all(ich+j-1) % ctp
            profiles_k(ich2+j-1) % cfraction         = profiles_k_all(ich+j-1) % cfraction
            Do  ii=1,coef%nlevels
               profiles_k(ich2+j-1) % t(ii)          = profiles_k_all(ich+j-1) % t(ii)
               profiles_k(ich2+j-1) % q(ii)          = profiles_k_all(ich+j-1) % q(ii)
               profiles_k(ich2+j-1) % o3(ii)         = profiles_k_all(ich+j-1) % o3(ii)
               profiles_k(ich2+j-1) % clw(ii)        = profiles_k_all(ich+j-1) % clw(ii)
            enddo
         End Do
      Else If (pol_id == 1) then
         ! Return average of V and H polarisation
         profiles_k(ich2) % s2m % t            = profiles_k_all(ich) % s2m % t + profiles_k_all(ich+1) % s2m % t
         profiles_k(ich2) % s2m % q            = profiles_k_all(ich) % s2m % q + profiles_k_all(ich+1) % s2m % q
         profiles_k(ich2) % s2m % p            = profiles_k_all(ich) % s2m % p + profiles_k_all(ich+1) % s2m % p
         profiles_k(ich2) % s2m % u            = profiles_k_all(ich) % s2m % u + profiles_k_all(ich+1) % s2m % u
         profiles_k(ich2) % s2m % v            = profiles_k_all(ich) % s2m % v + profiles_k_all(ich+1) % s2m % v
         profiles_k(ich2) % skin % t           = profiles_k_all(ich) % skin % t + profiles_k_all(ich+1) % skin % t
         profiles_k(ich2) % skin % fastem(1)   = profiles_k_all(ich) % skin % fastem(1) + profiles_k_all(ich+1) % skin % fastem(1)
         profiles_k(ich2) % skin % fastem(2)   = profiles_k_all(ich) % skin % fastem(2) + profiles_k_all(ich+1) % skin % fastem(2)
         profiles_k(ich2) % skin % fastem(3)   = profiles_k_all(ich) % skin % fastem(3) + profiles_k_all(ich+1) % skin % fastem(3)
         profiles_k(ich2) % skin % fastem(4)   = profiles_k_all(ich) % skin % fastem(4) + profiles_k_all(ich+1) % skin % fastem(4)
         profiles_k(ich2) % skin % fastem(5)   = profiles_k_all(ich) % skin % fastem(5) + profiles_k_all(ich+1) % skin % fastem(5)
         profiles_k(ich2) % ctp                = profiles_k_all(ich) % ctp + profiles_k_all(ich+1) % ctp
         profiles_k(ich2) % cfraction          = profiles_k_all(ich) % cfraction + profiles_k_all(ich+1) % cfraction
         Do  ii=1,coef%nlevels
            profiles_k(ich2) % t(ii)        = profiles_k_all(ich) % t(ii) + profiles_k_all(ich+1) % t(ii)
            profiles_k(ich2) % q(ii)        = profiles_k_all(ich) % q(ii) + profiles_k_all(ich+1) % q(ii)
            profiles_k(ich2) % o3(ii)       = profiles_k_all(ich) % o3(ii) + profiles_k_all(ich+1) % o3(ii)
            profiles_k(ich2) % clw(ii)      = profiles_k_all(ich) % clw(ii) + profiles_k_all(ich+1) % clw(ii)
         enddo
      Else
         !geom        => geometry( lprofiles(i) )
         emissfactor_v = pol_v(1,pol_id)+pol_v(2,pol_id)+pol_v(3,pol_id)
         emissfactor_h = pol_h(1,pol_id)+pol_h(2,pol_id)+pol_h(3,pol_id)
         profiles_k(ich2) % s2m % t     = profiles_k_all(ich) % s2m % t*emissfactor_v +&
                                             profiles_k_all(ich+1) % s2m % t*emissfactor_h
         profiles_k(ich2) % s2m % q     = profiles_k_all(ich) % s2m % q*emissfactor_v + &
                                             profiles_k_all(ich+1) % s2m % q*emissfactor_h
         profiles_k(ich2) % s2m % p     = profiles_k_all(ich) % s2m % p*emissfactor_v + &
                                             profiles_k_all(ich+1) % s2m % p*emissfactor_h
         profiles_k(ich2) % s2m % u     = profiles_k_all(ich) % s2m % u*emissfactor_v + &
                                             profiles_k_all(ich+1) % s2m % u*emissfactor_h
         profiles_k(ich2) % s2m % v     = profiles_k_all(ich) % s2m % v*emissfactor_v + &
                                             profiles_k_all(ich+1) % s2m % v*emissfactor_h
         profiles_k(ich2) % skin % t     = profiles_k_all(ich) % skin % t*emissfactor_v + &
                                             profiles_k_all(ich+1) % skin % t*emissfactor_h
         profiles_k(ich2) % skin % fastem(1)   = profiles_k_all(ich) % skin % fastem(1)*emissfactor_v + &
                                             profiles_k_all(ich+1) % skin % fastem(1)*emissfactor_h
         profiles_k(ich2) % skin % fastem(2)   = profiles_k_all(ich) % skin % fastem(2)*emissfactor_v + &
                                             profiles_k_all(ich+1) % skin % fastem(2)*emissfactor_h
         profiles_k(ich2) % skin % fastem(3)   = profiles_k_all(ich) % skin % fastem(3)*emissfactor_v + &
                                             profiles_k_all(ich+1) % skin % fastem(3)*emissfactor_h
         profiles_k(ich2) % skin % fastem(4)   = profiles_k_all(ich) % skin % fastem(4)*emissfactor_v + &
                                             profiles_k_all(ich+1) % skin % fastem(4)*emissfactor_h
         profiles_k(ich2) % skin % fastem(5)   = profiles_k_all(ich) % skin % fastem(5)*emissfactor_v + &
                                             profiles_k_all(ich+1) % skin % fastem(5)*emissfactor_h
         profiles_k(ich2) % ctp      = profiles_k_all(ich) % ctp*emissfactor_v + &
                                             profiles_k_all(ich+1) % ctp*emissfactor_h
         profiles_k(ich2) % cfraction     = profiles_k_all(ich) % cfraction*emissfactor_v + &
                                             profiles_k_all(ich+1) % cfraction*emissfactor_h
         Do  ii=1,coef%nlevels
            profiles_k(ich2) % t(ii)        = profiles_k_all(ich) % t(ii)*emissfactor_v +&
                                                profiles_k_all(ich+1) % t(ii)*emissfactor_h
            profiles_k(ich2) % q(ii)        = profiles_k_all(ich) % q(ii)*emissfactor_v + &
                                                profiles_k_all(ich+1) % q(ii)*emissfactor_h
            profiles_k(ich2) % o3(ii)        = profiles_k_all(ich) % o3(ii)*emissfactor_v + &
                                                profiles_k_all(ich+1) % o3(ii)*emissfactor_h
            profiles_k(ich2) % clw(ii)        = profiles_k_all(ich) % clw(ii)*emissfactor_v + &
                                                profiles_k_all(ich+1) % clw(ii)*emissfactor_h
         enddo
      End If
      ich2 = ich2 + npolar_return(pol_id)
   End Do
End Subroutine rttov_profout_k
